#include <xparameters.h>
#include <xstatus.h>
#include "peripheral.h"
#include "mac_phy_test.h"

//This is used for Digilant manufacture test, it is pass/fail only
//Return 1 means fail
//Return 0 means pass
int  emac_mfg_test(void)
{
  //0. Init the EMAC data structure
  XEmac Emac;
  XEmac *InstancePtr = &Emac;
  Xuint16 DeviceId = DEVICE_ID;
  
  if( emac_init(InstancePtr, DeviceId) )
    return 1;
  
  //1. Self test
  if( emac_selfTest(InstancePtr) )
    return 1;
  
  //2. Test the communication between EMAC and the PHY
  if(emac_phyTest(InstancePtr, 0) )
    return 1;
  
  //3. Test the normal function of EMAC and the PHY, external loopback plug
  //   must be plugged into the RJ45 socket
  FastEthernet = XFALSE;
  FullDuplex = XTRUE;
  
  if(emac_phyLoopBackTest(InstancePtr))
    return 1;
  
  return 0;
}


//Initialize the EMAC controller
int emac_init(XEmac *InstancePtr, Xuint16 DeviceId){
  
  XStatus Result;
  
  Result = XEmac_Initialize(InstancePtr, DeviceId);
  
  //Stop and re-init if EMAC has already been started
  if (Result == XST_DEVICE_IS_STARTED){
    (void)XEmac_Stop(InstancePtr);
    Result = XEmac_Initialize(InstancePtr, DeviceId);
  }
  
  //Check the result
  if (Result != XST_SUCCESS)
    return 1;
  
  return 0;
}

//Set the EMAC option
int emac_setMACOptions(XEmac *InstancePtr, Xuint32 Options)
{
  Xuint16 RegisterNum;
  Xuint16 PhyData;
  XStatus Result;
  
  //Stop EMAC 
  XEmac_Stop(InstancePtr);
  
  //Set the EMAC option, turn on full duplex if needed
  if(FullDuplex)
    Result = XEmac_SetOptions(InstancePtr, Options | XEM_FDUPLEX_OPTION);
  else
    Result = XEmac_SetOptions(InstancePtr, Options);
  
  //Check the result
  if (Result != XST_SUCCESS)
    return 1;
  
  return 0;
}

//EMAC self test
int emac_selfTest(XEmac *InstancePtr)
{
  //Set the EMAC option for self test
  Xuint32 Options;
  XStatus Result;
  
  Options = (XEM_UNICAST_OPTION |
             XEM_BROADCAST_OPTION |
             XEM_POLLED_OPTION |
             XEM_INSERT_PAD_OPTION |
             XEM_INSERT_FCS_OPTION |
             XEM_INSERT_ADDR_OPTION |
             XEM_OVWRT_ADDR_OPTION);
    
  if( emac_setMACOptions(InstancePtr, Options) )
    return 1;
  
  //Run self test
  Result = XEmac_SelfTest(InstancePtr);
  
  //Check result
    if (Result != XST_SUCCESS)
      return 1;
    
    return 0;
}

//Helper method: send an arbitrary length(46-1500) frame and check the
//received frame with the send frame
int emac_loopBackFrame(XEmac *InstancePtr, Xuint32 PayloadSize)
{
  if( emac_sendFrame(InstancePtr, PayloadSize, LocalAddress) == 0 ){
    if (emac_receiveFrame(InstancePtr, PayloadSize, XTRUE) == 0){
      return 0;
    }
  }
  return 1;
}

//Helper method: send an arbitrary length frame
int emac_sendFrame(XEmac *InstancePtr, Xuint32 PayloadSize, Xuint8 *DestAddress)
{
  Xuint32 Frame[XEM_MAX_FRAME_SIZE_IN_WORDS];
  Xuint8 *FramePtr;
  Xuint8 *AddrPtr = DestAddress;
  int Index;
  XStatus Result;
  
  FramePtr = (Xuint8 *)Frame;
  
  //Set the destination MAC address
  *FramePtr++ = *AddrPtr++;
  *FramePtr++ = *AddrPtr++;
  *FramePtr++ = *AddrPtr++;
  *FramePtr++ = *AddrPtr++;
  *FramePtr++ = *AddrPtr++;
  *FramePtr++ = *AddrPtr++;
  
  //Skip the source MAC address, the caller should be responsible
  //for setting the INSERT_SOURCE_ADDRESS and
  //OVERWRITE_SOURCE_ADDRESS option
  FramePtr += XEM_MAC_ADDR_SIZE;
  
  //Set up the type/length field - be sure its in network order
  *((Xuint16 *)FramePtr) = PayloadSize;
  FramePtr++;
  FramePtr++;
  
  //Now fill the payload with known pattern
  for (Index = 0; Index < PayloadSize; Index++)
    *FramePtr++ = (Xuint8)Index;
  
  //Send the frame
  Result = XEmac_PollSend(InstancePtr, (Xuint8 *)Frame, PayloadSize + XEM_HDR_SIZE);
  if (Result != XST_SUCCESS)
    return Result;
  else
    return 0;
}

//Helper method: receive an arbitrary length frame
int emac_receiveFrame(XEmac *InstancePtr, Xuint32 PayloadSize, Xboolean Verify)
{
  //This is assuming that the payload size is larger than 46 bytes.
  Xuint32 Frame[XEM_MAX_FRAME_SIZE_IN_WORDS];
  Xuint8 *FramePtr;
  XStatus Result;
  Xuint32 RecvFrameLength;
  
  RecvFrameLength = XEM_MAX_FRAME_SIZE; //You have to do this before receiving
  Result = XEmac_PollRecv(InstancePtr, (Xuint8 *)Frame, &RecvFrameLength);
  if (Result == XST_SUCCESS){
    int Index;
    
    /* Verify length, which should be the payload plus overhead */
    if (RecvFrameLength != (PayloadSize + XEM_HDR_SIZE + XEM_TRL_SIZE))
      return 1;
    
    /* Verify the frame contents */
    if (Verify){
      FramePtr = (Xuint8 *)Frame;
      FramePtr += XEM_HDR_SIZE;
      for (Index = 0; Index < PayloadSize; Index++){
        if (*FramePtr++ != (Xuint8)Index)
          return 2;
      }
    }
    return 0;
  }
  else
    return Result;
}

//PHY register read and write test
int emac_phyTest(XEmac *InstancePtr, Xuint8 PhyAddress)
{
  Xuint16 PHY_ID1 = 0x0013;
  Xuint16 PHY_ID2 = 0x78E2;
  
  Xuint16 PhyData;
  const char *PhyName = "LXT972A";
  Xboolean success = XTRUE;
  
  
  //Read control register
  if (XEmac_PhyRead(InstancePtr, PhyAddress, 0, &PhyData))
    success = XFALSE;
  
  //Reset PHY, read control register and status register
  if( XEmac_PhyWrite(InstancePtr, PhyAddress, 0, 0x8000) )
    success = XFALSE;
  
  //Read control register
  if ( XEmac_PhyRead(InstancePtr, PhyAddress, 0, &PhyData) )
    success = XFALSE;
  
  //Read status register
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 1, &PhyData) )
    success = XFALSE;
  
  //This is a hack, you have to delay some time before reading the ID reg
  {
    int delay;
    for(delay=0; delay<100000; delay++);
  }
  
  //Check PHY identification register
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 2, &PhyData) )
    success = XFALSE;
  
  if (PhyData != PHY_ID1)
    success = XFALSE;
  
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 3, &PhyData) )
    success = XFALSE;
  
  if (PhyData != PHY_ID2)
    success = XFALSE;
  
  // Auto Negotiation advertisement register (ANAR)
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 4, &PhyData) )
    success = XFALSE;
  
  // Auto Negotiation link partner ability (ANLPAR)
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 5, &PhyData) )
    success = XFALSE;
  
  // Auto Negotiation expansion
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 6, &PhyData) )
    success = XFALSE;
  
  // Mirror register
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 16, &PhyData) )
    success = XFALSE;
  
  // Interrupt enable register
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 17, &PhyData) )
    success = XFALSE;
  
  // Interrupt status register
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 18, &PhyData) )
    success = XFALSE;
  
  // Configuration register
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 19, &PhyData) )
    success = XFALSE;
  
  // Chip status register
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 20, &PhyData) )
    success = XFALSE;
  
  // Enable interrupt register
  if( XEmac_PhyWrite(InstancePtr, PhyAddress, 17, 0x000C))
    success = XFALSE;
  
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 17, &PhyData) )
    success = XFALSE;
  
  // Enable loopback, read back, disable loopback
  if( XEmac_PhyWrite(InstancePtr, PhyAddress, 19, 0x8000))
    success = XFALSE;
  
  if( XEmac_PhyRead(InstancePtr, PhyAddress, 19, &PhyData) )
    success = XFALSE;
  
  if( XEmac_PhyWrite(InstancePtr, PhyAddress, 19, 0x0000))
    success = XFALSE;
  
  if(success){
    print("                 \r"); //This is a hack, you have to print sth here, otherwise, the next test will fail.
    return 0;
  }else
    return 1;
}

//PHY loop back test
int emac_phyLoopBackTest(XEmac *InstancePtr)
{
  Xuint32 Options;
  
  emac_setPHYOptions(InstancePtr, XFALSE);
  //Force link up
  XEmac_PhyWrite(InstancePtr, 0, 16, 0x4000);
  
  XEmac_Stop(InstancePtr);
  
  //Set the local MAC address
  if( XEmac_SetMacAddress(InstancePtr, LocalAddress) )
    return 1;
  
  Options = (XEM_POLLED_OPTION |
             XEM_UNICAST_OPTION |
             XEM_INSERT_PAD_OPTION |
             XEM_INSERT_FCS_OPTION |
             XEM_INSERT_ADDR_OPTION |
             XEM_OVWRT_ADDR_OPTION);
  
  if (emac_setMACOptions(InstancePtr, Options))
    return 1;
  
  if( XEmac_Start(InstancePtr) )
    return 1;
  
  //Loop back test from 46 bytes to 1500 bytes
  {
    int Size;
    Xboolean success = XTRUE;
    for (Size=46; Size <= XEM_MTU; Size ++)
      if(emac_loopBackFrame(InstancePtr, Size)){
        success = XFALSE;
        break;
      }
    
    if(!success)
      return 1;
  }
  
  //Send two frames successively, and then receive them
  {
    Xboolean success = XFALSE;
    
    if( emac_sendFrame(InstancePtr, 60, LocalAddress) == 0)
      if ( emac_sendFrame(InstancePtr, 120, LocalAddress) == 0)
        if ( emac_receiveFrame(InstancePtr, 60, XTRUE) == 0)
          if (emac_receiveFrame(InstancePtr, 120, XTRUE) == 0)
            success = XTRUE;
    if(!success)
      return 1;
  }
  
  //Send 16 frames successively (the length/status FIFO depths) and receive them
  {
    int i;
    Xboolean send_success = XTRUE;
    Xboolean rec_success = XTRUE;
    
    for (i=60; i < 76; i++)
      if(emac_sendFrame(InstancePtr, i, LocalAddress)){
        send_success = XFALSE;
        break;
      }
    if(!send_success)
      return 1;
    else{
      for (i=60; i < 76; i++)
        if(emac_receiveFrame(InstancePtr, i, XTRUE)){
          rec_success = XFALSE;
          break;
        }
      if(!rec_success)
        return 1;
    }
  }
  
  //Send three frames successively(1500,1500, 1000) and receive them.
  {
    Xboolean success = XFALSE;
    
    if( emac_sendFrame(InstancePtr, 60, LocalAddress) == 0)
      if ( emac_sendFrame(InstancePtr, 120, LocalAddress) == 0)
        if ( emac_sendFrame(InstancePtr, 120, LocalAddress) == 0)
          if ( emac_receiveFrame(InstancePtr, 60, XTRUE) == 0)
            if (emac_receiveFrame(InstancePtr, 120, XTRUE) == 0)
              if (emac_receiveFrame(InstancePtr, 120, XTRUE) == 0)
                success = XTRUE;
    if(!success)
      return 1;
  }
  
  return 0;
}

//Set the PHY option
int emac_setPHYOptions(XEmac *InstancePtr, Xboolean loopBack)
{
  Xuint16 PhyData;
  PhyData = 0;
  
  //Reset PHY, read control register and status register
  if( XEmac_PhyWrite(InstancePtr, 0, 0, 0x8000) )
    return 1;
  
  xil_printf("                     \r"); //This is a hack, you have to print sth here, otherwise, the following will fail.
  
  //Set the PHY into full duplex if needed
  if (XEmac_PhyRead(InstancePtr, 0, 0, &PhyData))
    return 1;
  
  PhyData &= 0xEFFF; //Disable auto negotiation
  
  if(FullDuplex)
    PhyData |= 0x0100;
  else
    PhyData &= 0xFEFF;
  
  if(FastEthernet)
    PhyData |= 0x2000;
  else
    PhyData &= 0xDFFF;
  
  if(loopBack)
    PhyData |= 0x4000;
  else
    PhyData &= 0xBFFF;
  
  if (XEmac_PhyWrite(InstancePtr, 0, 0, PhyData))
    return 1;
  
  
  if (XEmac_PhyRead(InstancePtr, 0, 17, &PhyData))
    return 1;
  
  return 0;
}
